/*
 * CPM client to access CPM servers.
 *
 * Copyright (C) 2018, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/cxx/ipc_stream>
#include <l4/re/env>

#include "cpm-client.h"
#include "cpm-ops.h"

int
Cpm_device_interface::get_lcd_pixel_frequency(uint32_t *frequency) throw()
{
  L4::Ipc::Iostream s(l4_utcb());
  int err = l4_error(s.call(cap(), Cpm_op_get_lcd_pixel_frequency));
  uint32_t _frequency;

  if (err)
    return err;

  s >> _frequency;
  *frequency = _frequency;
  return L4_EOK;
}

int
Cpm_device_interface::set_hdmi_frequency(uint32_t frequency) throw()
{
  L4::Ipc::Iostream s(l4_utcb());

  s << frequency;

  return l4_error(s.call(cap(), Cpm_op_set_hdmi_frequency));
}

int
Cpm_device_interface::start_hdmi() throw()
{
  L4::Ipc::Iostream s(l4_utcb());

  return l4_error(s.call(cap(), Cpm_op_start_hdmi));
}

int
Cpm_device_interface::stop_hdmi() throw()
{
  L4::Ipc::Iostream s(l4_utcb());

  return l4_error(s.call(cap(), Cpm_op_stop_hdmi));
}

int
Cpm_device_interface::set_lcd_frequencies(uint32_t pclk, uint8_t multiplier) throw()
{
  L4::Ipc::Iostream s(l4_utcb());

  s << pclk << multiplier;

  return l4_error(s.call(cap(), Cpm_op_set_lcd_frequencies));
}

int
Cpm_device_interface::start_lcd() throw()
{
  L4::Ipc::Iostream s(l4_utcb());

  return l4_error(s.call(cap(), Cpm_op_start_lcd));
}

int
Cpm_device_interface::stop_lcd() throw()
{
  L4::Ipc::Iostream s(l4_utcb());

  return l4_error(s.call(cap(), Cpm_op_stop_lcd));
}

int
Cpm_device_interface::update_output_frequency() throw()
{
  L4::Ipc::Iostream s(l4_utcb());

  return l4_error(s.call(cap(), Cpm_op_update_output_frequency));
}
